/*
 * Author:          Patrick-Christopher Mattulat
 * Company:         Lynar Studios
 * E-Mail:          webmaster@lynarstudios.com
 * Created:         2020-08-09
 * Changed:         2024-05-31
 *
 * */

#ifndef LS_STD_BOOLEAN_HPP
#define LS_STD_BOOLEAN_HPP

#include <ls-std/core/Class.hpp>
#include <ls-std/core/definition.hpp>
#include <ls-std/core/interface/IBoxing.hpp>
#include <sstream>

namespace ls::std::boxing
{
  ls_std_class Boolean final : public core::Class, public core::interface_type::IBoxing
  {
    public:

      explicit Boolean(bool _value);
      Boolean();
      ~Boolean() noexcept override;

      // assignment operators

      Boolean &operator=(int _value);
      Boolean &operator=(bool _value);

      // stream operators

      friend ::std::ostream &operator<<(::std::ostream &_outputStream, const Boolean &_boolean)
      {
        _outputStream << _boolean._toString();
        return _outputStream;
      }

      // logical operators

      friend bool operator!(const Boolean &_boolean)
      {
        return !_boolean.value;
      }

      bool operator&&(const Boolean &_boolean) const;
      bool operator&&(bool _value) const;
      bool operator&&(int _value) const;
      bool operator||(const Boolean &_boolean) const;
      bool operator||(bool _value) const;
      bool operator||(int _value) const;
      // INFO: operator ^ can not be taken for XOR, since it's not possible to implement it respecting commutative law

      // implementation

      void parse(const ::std::string &_parseText) override;
      [[nodiscard]] ::std::string toString() override;

      // additional functionality

      [[nodiscard]] bool getValue() const;
      [[nodiscard]] static bool XOR(const Boolean &_leftExpression, const Boolean &_rightExpression);
      [[nodiscard]] static bool XOR(const Boolean &_leftExpression, bool _rightExpression);
      [[nodiscard]] static bool XOR(bool _leftExpression, const Boolean &_rightExpression);
      [[nodiscard]] static bool XOR(bool _leftExpression, bool _rightExpression);

    private:

      bool value{};

      const ::std::string FALSE_STRING = "false";
      const ::std::string TRUE_STRING = "true";

      [[nodiscard]] static ::std::string _fetchClassName();
      [[nodiscard]] ::std::string _toString() const;
  };
}

#endif
